@@ -4,8 +4,9 @@
  * No warranty is given; refer to the file DISCLAIMER.PD within this package.
  */
 
+#include <crtdefs.h>
 #include <sect_attribs.h>
-#include <internal.h>
+#include <corecrt_startup.h>
 
 __declspec(dllimport) int __lconv_init (void);
 
