@@ -9,6 +9,10 @@
 #define _DLL
 #endif
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #define SPECIAL_CRTEXE
 
 #include <oscalls.h>
@@ -21,12 +25,16 @@
 #include <sect_attribs.h>
 #include <locale.h>
 
+#if defined(__SEH__) && (!defined(__clang__) || __clang_major__ >= 7)
+#define SEH_INLINE_ASM
+#endif
+
 #ifndef __winitenv
 extern wchar_t *** __MINGW_IMP_SYMBOL(__winitenv);
 #define __winitenv (* __MINGW_IMP_SYMBOL(__winitenv))
 #endif
 
-#if !defined(__initenv) && !defined(__arm__)
+#if !defined(__initenv) && !defined(__arm__) && !defined(__aarch64__)
 extern char *** __MINGW_IMP_SYMBOL(__initenv);
 #define __initenv (* __MINGW_IMP_SYMBOL(__initenv))
 #endif
@@ -60,12 +68,16 @@ extern _CRTALLOC(".CRT$XIZ") _PIFV __xi_z[];
 extern _CRTALLOC(".CRT$XCA") _PVFV __xc_a[];
 extern _CRTALLOC(".CRT$XCZ") _PVFV __xc_z[];
 
+#ifndef HAVE_CTOR_LIST
+__attribute__ (( __section__ (".ctors"), __used__ , aligned(sizeof(void *)))) const void * __CTOR_LIST__ = (void *) -1;
+__attribute__ (( __section__ (".dtors"), __used__ , aligned(sizeof(void *)))) const void * __DTOR_LIST__ = (void *) -1;
+__attribute__ (( __section__ (".ctors.99999"), __used__ , aligned(sizeof(void *)))) const void * __CTOR_END__ = (void *) 0;
+__attribute__ (( __section__ (".dtors.99999"), __used__ , aligned(sizeof(void *)))) const void * __DTOR_END__ = (void *) 0;
+#endif
+
 /* TLS initialization hook.  */
 extern const PIMAGE_TLS_CALLBACK __dyn_tls_init_callback;
 
-extern _PVFV *__onexitbegin;
-extern _PVFV *__onexitend;
-
 extern int mingw_app_type;
 
 HINSTANCE __mingw_winmain_hInstance;
@@ -130,9 +142,8 @@ pre_c_init (void)
     __set_app_type(_GUI_APP);
   else
     __set_app_type (_CONSOLE_APP);
-  __onexitbegin = __onexitend = (_PVFV *) _encode_pointer ((_PVFV *)(-1));
 
-  * __MINGW_IMP_SYMBOL(_fmode) = _fmode;
+  * __p__fmode() = _fmode;
   * __MINGW_IMP_SYMBOL(_commode) = _commode;
 
 #ifdef WPRFLAG
@@ -170,7 +181,7 @@ int WinMainCRTStartup (void);
 int WinMainCRTStartup (void)
 {
   int ret = 255;
-#if defined(__SEH__) && !defined(__clang__)
+#ifdef SEH_INLINE_ASM
   asm ("\t.l_startw:\n"
     "\t.seh_handler __C_specific_handler, @except\n"
     "\t.seh_handlerdata\n"
@@ -182,7 +193,7 @@ int WinMainCRTStartup (void)
   mingw_app_type = 1;
   __security_init_cookie ();
   ret = __tmainCRTStartup ();
-#if defined(__SEH__) && !defined(__clang__)
+#ifdef SEH_INLINE_ASM
   asm ("\tnop\n"
     "\t.l_endw: nop\n");
 #endif
@@ -198,7 +209,7 @@ int __mingw_init_ehandler (void);
 int mainCRTStartup (void)
 {
   int ret = 255;
-#if defined(__SEH__) && !defined(__clang__)
+#ifdef SEH_INLINE_ASM
   asm ("\t.l_start:\n"
     "\t.seh_handler __C_specific_handler, @except\n"
     "\t.seh_handlerdata\n"
@@ -210,7 +221,7 @@ int mainCRTStartup (void)
   mingw_app_type = 0;
   __security_init_cookie ();
   ret = __tmainCRTStartup ();
-#if defined(__SEH__) && !defined(__clang__)
+#ifdef SEH_INLINE_ASM
   asm ("\tnop\n"
     "\t.l_end: nop\n");
 #endif
@@ -218,6 +229,11 @@ int mainCRTStartup (void)
 }
 
 static
+#if defined(__i386__) || defined(_X86_)
+/* We need to make sure that we align the stack to 16 bytes for the sake of SSE
+   opts in main or in functions called main.  */
+__attribute__((force_align_arg_pointer))
+#endif
 __declspec(noinline) int
 __tmainCRTStartup (void)
 {
@@ -226,17 +242,6 @@ __tmainCRTStartup (void)
   WINBOOL inDoubleQuote = FALSE;
   memset (&StartupInfo, 0, sizeof (STARTUPINFO));
 
-#if defined(__i386__) || defined(_X86_)
-  /* We need to make sure that this function is build with frame-pointer
-     and that we align the stack to 16 bytes for the sake of SSE ops in main
-     or in functions inlined into main.  */
-  lpszCommandLine = (_TCHAR *) alloca (32);
-  memset (lpszCommandLine, 0xcc, 32);
-#ifdef __GNUC__
-  asm  __volatile__  ("andl $-16, %%esp" : : : "%esp");
-#endif
-#endif /* defined(__i386__) || defined(_X86_) */
-
   if (mingw_app_type)
     GetStartupInfo (&StartupInfo);
   {
@@ -279,7 +284,7 @@ __tmainCRTStartup (void)
     
     _pei386_runtime_relocator ();
     __mingw_oldexcpt_handler = SetUnhandledExceptionFilter (_gnu_exception_handler);
-#ifdef _WIN64
+#ifdef __x86_64__
     __mingw_init_ehandler ();
 #endif
     _set_invalid_parameter_handler (__mingw_invalidParameterHandler);
@@ -328,7 +333,7 @@ __tmainCRTStartup (void)
        gcc inserts this call automatically for a function called main, but not for wmain.  */
     mainret = wmain (argc, argv, envp);
 #else
-#ifndef __arm__
+#if !defined(__arm__) && !defined(__aarch64__)
     __initenv = envp;
 #endif
     mainret = main (argc, argv, envp);
@@ -428,3 +433,10 @@ static void duplicate_ppstrings (int ac, char ***av)
 	*av = n;
 }
 #endif
+
+int __cdecl atexit (_PVFV func)
+{
+    return _onexit((_onexit_t)func) ? 0 : -1;
+}
+
+char __mingw_module_is_dll = 0;
