@@ -1,6 +1,6 @@
 /*
     gendef - Generate list of exported symbols from a Portable Executable.
-    Copyright (C) 2009-2014  mingw-w64 project
+    Copyright (C) 2009-2016  mingw-w64 project
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -658,7 +658,10 @@ dump_def (void)
       if (exp->name[0] == 0)
         fprintf (fp, "ord_%u", (unsigned int) exp->ord);
       else
-        fprintf (fp, "%s", exp->name);
+        {
+          const char *quote = strchr (exp->name, '.') ? "\"" : "";
+          fprintf (fp, "%s%s%s", quote, exp->name, quote);
+        }
       if (exp->name[0] == '?' && exp->name[1] == '?')
         {
           if (!strncmp (exp->name, "??_7", 4))
