@@ -25,7 +25,7 @@ extern "C" {
 #endif
 
 #ifndef _NORMALIZE_
-# define WINNORMALIZEAPI DECLSPEC_IMPORT
+# define WINNORMALIZEAPI DECLSPEC_HIDDEN
 #else
 # define WINNORMALIZEAPI
 #endif
@@ -169,10 +169,12 @@ extern "C" {
 #define LOCALE_USE_CP_ACP             0x40000000
 #define LOCALE_RETURN_NUMBER          0x20000000
 #define LOCALE_RETURN_GENITIVE_NAMES  0x10000000
+#define LOCALE_ALLOW_NEUTRAL_NAMES    0x08000000
 
 /* Locale information types */
 #define LOCALE_ILANGUAGE            0x0001
 #define LOCALE_SLANGUAGE            0x0002
+#define LOCALE_SLOCALIZEDDISPLAYNAME 0x0002
 #define LOCALE_SENGLANGUAGE         0x1001
 #define LOCALE_SENGLISHLANGUAGENAME 0x1001
 #define LOCALE_SABBREVLANGNAME      0x0003
@@ -322,17 +324,13 @@ extern "C" {
 #define LOCALE_IDIGITSUBSTITUTION     0x1014
 
 /* Locale name special values */
-#if defined(__GNUC__)
-# define LOCALE_NAME_INVARIANT      (const WCHAR []){ 0 }
-#elif defined(_MSC_VER)
+#if defined(_MSC_VER) || defined(__MINGW32__)
 # define LOCALE_NAME_INVARIANT      L""
 #else
 static const WCHAR LOCALE_NAME_INVARIANT[] = { 0 };
 #endif
 
-#if defined(__GNUC__)
-# define LOCALE_NAME_SYSTEM_DEFAULT      (const WCHAR []){'!','s','y','s','-','d','e','f','a','u','l','t','-','l','o','c','a','l','e',0}
-#elif defined(_MSC_VER)
+#if defined(_MSC_VER) || defined(__MINGW32__)
 # define LOCALE_NAME_SYSTEM_DEFAULT      L"!sys-default-locale"
 #else
 static const WCHAR LOCALE_NAME_SYSTEM_DEFAULT[] = {'!','s','y','s','-','d','e','f','a','u','l','t','-','l','o','c','a','l','e',0};
@@ -551,6 +549,8 @@ static const WCHAR LOCALE_NAME_SYSTEM_DEFAULT[] = {'!','s','y','s','-','d','e','
 #define CAL_GREGORIAN_ARABIC       10
 #define CAL_GREGORIAN_XLIT_ENGLISH 11
 #define CAL_GREGORIAN_XLIT_FRENCH  12
+#define CAL_PERSIAN                22
+#define CAL_UMALQURA               23
 
 /* EnumCalendarInfo Flags */
 #define ENUM_ALL_CALENDARS 0xffffffff /* Enumerate all calendars within a locale */
@@ -833,6 +833,7 @@ enum SYSGEOTYPE
 
 enum SYSGEOCLASS
 {
+    GEOCLASS_ALL = 0,
     GEOCLASS_REGION = 14,
     GEOCLASS_NATION = 16
 };
