@@ -30,7 +30,8 @@ do
 	fi
 done
 
-for f in basetsd.h guiddef.h ndrtypes.h poppack.h pshpack1.h pshpack2.h pshpack4.h pshpack8.h winbase.h windef.h winerror.h winnls.h winnt.h winnt.rh
+for f in basetsd.h guiddef.h excpt.h ndrtypes.h poppack.h pshpack1.h pshpack2.h pshpack4.h pshpack8.h threadpoolapiset.h \
+        winbase.h windef.h winerror.h winnls.h winnt.h winnt.rh synchapi.h
 do
 	cp $WINE_DIR/include/$f include
 done
@@ -40,7 +41,7 @@ do
 	cp $WINE_DIR/include/wine/$f include/wine
 done
 
-for f in getopt.c getopt1.c
+for f in getopt.c
 do
 	cp $WINE_DIR/libs/port/$f src/port/
 done
